#!/usr/bin/perl


##########################################################
# Converts simple formatted ASCII Texts to DocBook XML
#
# (c) Thomas Weber 2002
#
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Please see the file 'LICENSE.txt' for detailed copyright description 
# and the disclaimer  
##########################################################

##########################################
# Global vars and imports 
#{{{######################################

my @_CURRENTBLOCKS;		# contains a list of the current parent blocks (including the active one)

sub getCurrentblock {
	return $_CURRENTBLOCKS[$#_CURRENTBLOCKS];	
}

require "output.pl";
use blocks;

#}}}

##########################################
# Handle a new Section
#{{{######################################
sub doSection {
	my ($indents,$depth,$title)=@_;
	my $block;
	my @sections;
	my $done;
	
	# open new sublevel
	$block=Section->new($indents,$depth,$title);
	doBlockend($block);
	$block->begin();
	
	push @_CURRENTBLOCKS,$block;

	
}
#}}}

##########################################
# Handle empty line
#{{{######################################
sub doEmpty {
	
	doBlockend();
	
}
#}}}

##########################################
# Handle block end
# {{{ ####################################
sub doBlockend {

	my $newblock=shift;
	my $lastblock;
	my $success;
	
	do {
		$success=0;
		$lastblock=getCurrentblock();
		
		return if (!$lastblock);
			
		if ( $lastblock->end($newblock) ) {
			pop @_CURRENTBLOCKS;
			$success=1;
		}
		
	} while ($success);
	
	
}

#}}}

##########################################
# Handle a textline
# {{{ ####################################

sub doText {
	
	my $indents=shift;
	my $text=shift;
	my $lastblock=getCurrentblock();
	my $block;
	
	if (!$STARTDOC) {
		output::startDocument($text);
		$STARTDOC=1;
		return;
	}
	
	if (ref($lastblock) ne "Paragraph" && ref($lastblock) ne "ListItem" && ref($lastblock) ne "VarListItem" ) { 
		# begin a new paragraph
		$block=Paragraph->new($indents);
		doBlockend($block);
		$block->begin($text);
		push @_CURRENTBLOCKS,$block;
		
	} else {
		# Output the text
		output::textOutput($text);
	}
	
	
}

# }}}

##########################################
# Handle item line
# {{{ ####################################

sub doItem {
	
	my $indents=shift;
	my $text=shift;
	my $block;
	my $doitem;
	my $dolist;
	my $sublist;
	
	
	$block=ListItem->new($indents);
	
	doBlockend($block);
	
	# check if the parent is a list
	my $lastblock=getCurrentblock();
	
	if (ref($lastblock) eq "List") {
		# already on a list? 
		
		# check indentation and create new sublist as needed
		if ($indents > $lastblock->{indent}) {
			$dolist=1;
			$sublist=1;
			
		}
		
		$doitem=1;
	
	} else {
		# create a new list
		$dolist=1;
		$doitem=1;
	}

	if ($dolist) {
		# begin new list
		$block=List->new($indents);
		$block->{sublist}=1 if ($sublist);
		$block->begin();
		push @_CURRENTBLOCKS,$block;
	}
	
	if ($doitem) {	
		# create the item
		$block=ListItem->new($indents);
		$block->begin($text);
		push @_CURRENTBLOCKS,$block;
	}
	
	
}
# }}}

##########################################
# Handle varitem line
# {{{ ####################################

sub doVarItem {
	
	my $indents=shift;
	my $term=shift;
	my $text=shift;
	my $block;
	my $doitem;
	my $dolist;
	my $sublist;
	
	
	$block=ListItem->new($indents);
	
	doBlockend($block);
	
	# check if the parent is a list
	my $lastblock=getCurrentblock();
	
	if (ref($lastblock) eq "VarList") {
		# already on a list? 
		
		# check indentation and create new sublist as needed
		if ($indents > $lastblock->{indent}) {
			$dolist=1;
			$sublist=1;
			
		}
		
		$doitem=1;
	
	} else {
		# create a new list
		$dolist=1;
		$doitem=1;
	}

	if ($dolist) {
		# begin new list
		$block=VarList->new($indents);
		$block->{sublist}=1 if ($sublist);
		$block->begin();
		push @_CURRENTBLOCKS,$block;
	}
	
	if ($doitem) {	
		# create the item
		$block=VarListItem->new($indents);
		$block->begin($term,$text);
		push @_CURRENTBLOCKS,$block;
	}
	
	
}
# }}}


##########################################
##########################################
# MAIN
#{{{######################################
$filename = shift or die "Usage: $0 <filename> \n";

my $tagmode;

open IN,"<$filename" or die "Can't open file $filename \n";

while ($line=<IN>) {
	
	# {{{ Tagblocks
	if ($line =~ /^(\s*)\<(\/?\w+)\s*.*\>(.*)/) {
		# arbitrary docbook tag
		$indents=getindents($1);
		$tagname=$2;
		$text=$line;
		$tagname=lc($tagname);
		
		
		if ($tagmode) {
			
			if ('/'.$tagmode eq $tagname ) {
				$tagmode=0;
				output::endPassthrough($tagname,$line);
			} else {
				output::PassthroughOutput($tagmode,$line);	
			}
			
		} else {
			
			$tagmode=lc($tagname);
			output::PassthroughOutput($tagmode,$line);
		}
		
		next;
	} 
		
	if ($tagmode) {
		output::PassthroughOutput($tagmode,$line);
		next;
	}
	# }}}
	
	chop $line;
	
	if ($line =~ /^\s*$/) {
		# identify empty line
		doEmpty();
	} elsif ($line =~ /^(\s*)([\*|\d+\.)]+)\s+(.*)/) {
		# identify section		
		$indents=getindents($1);
		
		$depth=$2;
		$title=$3;
		
		if ($depth =~ /\*+/) {
			# '*' is used to mark sections
			$depth=length($depth);
		} else {
			# '1.1.1' is used to mark sections
			@list=split /\./,$depth;
			print STDERR $depth."\n";
			print STDERR "=".($#list+1)."\n";
			$depth=$#list+1;
		}
		doSection($indents,$depth,$title);
	
	} elsif ($line =~ /^(\s*)[-|o|\+|=]\s+(.*)/) {
		# identify list item
		$indents=getindents($1);
		$text=$2;
		doItem($indents,$text);
		
	} elsif ($line =~ /^(\s*)\[(.*)\](\s+(.*))?/) {
		# identify varlist
		$indents=getindents($1);
		$term=$2;
		$text=$4;
		doVarItem($indents,$term,$text);
		
	}  elsif ($line =~ /^(\s*)(.*)/) {
		# this is text
		$indents=getindents($1);
		$text=$2;
		doText($indents,$text);
	}
	
	}
close IN;
doBlockend("ENDDOCUMENT");

output::endDocument();
#}}}

sub getindents {
	my $indents=shift;
	
	$indents=$1;
	$indents=~s/\t/_/g;
	$indents=~s/\s//g;
	$indents=length($indents);

	return $indents;	
}

# :folding=explicit:
# :collapseFolds=1:	
